@@ -2,7 +2,7 @@
  *  @author : Paul Taylor
  *  @author : Eric Farng
  *
- *  Version @version:$Id: ID3v1Tag.java,v 1.28 2008/12/10 13:14:27 paultaylor Exp $
+ *  Version @version:$Id: ID3v1Tag.java,v 1.35 2009/11/12 15:42:57 paultaylor Exp $
  *
  *  MusicTag Copyright (C)2003,2004
  *
@@ -27,8 +27,6 @@ import org.jaudiotagger.audio.generic.Utils;
 import org.jaudiotagger.audio.mp3.MP3File;
 import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.tag.*;
-import org.jaudiotagger.tag.id3.framebody.FrameBodyPIC;
-import org.jaudiotagger.tag.id3.valuepair.ImageFormats;
 import org.jaudiotagger.tag.datatype.Artwork;
 import org.jaudiotagger.tag.reference.GenreTypes;
 
@@ -47,17 +45,17 @@ import java.util.regex.Matcher;
  */
 public class ID3v1Tag extends AbstractID3v1Tag implements Tag
 {
-    static EnumMap<TagFieldKey, ID3v1FieldKey> tagFieldToID3v1Field = new EnumMap<TagFieldKey, ID3v1FieldKey>(TagFieldKey.class);
+    static EnumMap<FieldKey, ID3v1FieldKey> tagFieldToID3v1Field = new EnumMap<FieldKey, ID3v1FieldKey>(FieldKey.class);
 
     static
     {
-        tagFieldToID3v1Field.put(TagFieldKey.ARTIST, ID3v1FieldKey.ARTIST);
-        tagFieldToID3v1Field.put(TagFieldKey.ALBUM, ID3v1FieldKey.ALBUM);
-        tagFieldToID3v1Field.put(TagFieldKey.TITLE, ID3v1FieldKey.TITLE);
-        tagFieldToID3v1Field.put(TagFieldKey.TRACK, ID3v1FieldKey.TRACK);
-        tagFieldToID3v1Field.put(TagFieldKey.YEAR, ID3v1FieldKey.YEAR);
-        tagFieldToID3v1Field.put(TagFieldKey.GENRE, ID3v1FieldKey.GENRE);
-        tagFieldToID3v1Field.put(TagFieldKey.COMMENT, ID3v1FieldKey.COMMENT);
+        tagFieldToID3v1Field.put(FieldKey.ARTIST, ID3v1FieldKey.ARTIST);
+        tagFieldToID3v1Field.put(FieldKey.ALBUM, ID3v1FieldKey.ALBUM);
+        tagFieldToID3v1Field.put(FieldKey.TITLE, ID3v1FieldKey.TITLE);
+        tagFieldToID3v1Field.put(FieldKey.TRACK, ID3v1FieldKey.TRACK);
+        tagFieldToID3v1Field.put(FieldKey.YEAR, ID3v1FieldKey.YEAR);
+        tagFieldToID3v1Field.put(FieldKey.GENRE, ID3v1FieldKey.GENRE);
+        tagFieldToID3v1Field.put(FieldKey.COMMENT, ID3v1FieldKey.COMMENT);
     }
 
     //For writing output
@@ -141,11 +139,11 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     {
         super(copyObject);
 
-        this.album = new String(copyObject.album);
-        this.artist = new String(copyObject.artist);
-        this.comment = new String(copyObject.comment);
-        this.title = new String(copyObject.title);
-        this.year = new String(copyObject.year);
+        this.album = copyObject.album;
+        this.artist = copyObject.artist;
+        this.comment = copyObject.comment;
+        this.title = copyObject.title;
+        this.year = copyObject.year;
         this.genre = copyObject.genre;
     }
 
@@ -167,11 +165,11 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
             {
                 convertedTag = new ID3v11Tag(mp3tag);
             }
-            this.album = new String(convertedTag.album);
-            this.artist = new String(convertedTag.artist);
-            this.comment = new String(convertedTag.comment);
-            this.title = new String(convertedTag.title);
-            this.year = new String(convertedTag.year);
+            this.album = convertedTag.album;
+            this.artist = convertedTag.artist;
+            this.comment = convertedTag.comment;
+            this.title = convertedTag.title;
+            this.year = convertedTag.year;
             this.genre = convertedTag.genre;
         }
     }
@@ -211,7 +209,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         this(file, "");
     }
 
-    public void add(TagField field)
+    public void addField(TagField field)
     {
         //TODO
     }
@@ -219,27 +217,27 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     public List<TagField> get(String id)
     {
 
-        if (TagFieldKey.ARTIST.name().equals(id))
+        if (FieldKey.ARTIST.name().equals(id))
         {
             return getArtist();
         }
-        else if (TagFieldKey.ALBUM.name().equals(id))
+        else if (FieldKey.ALBUM.name().equals(id))
         {
             return getAlbum();
         }
-        else if (TagFieldKey.TITLE.name().equals(id))
+        else if (FieldKey.TITLE.name().equals(id))
         {
             return getTitle();
         }
-        else if (TagFieldKey.GENRE.name().equals(id))
+        else if (FieldKey.GENRE.name().equals(id))
         {
             return getGenre();
         }
-        else if (TagFieldKey.YEAR.name().equals(id))
+        else if (FieldKey.YEAR.name().equals(id))
         {
             return getYear();
         }
-        else if (TagFieldKey.COMMENT.name().equals(id))
+        else if (FieldKey.COMMENT.name().equals(id))
         {
             return getComment();
         }
@@ -258,18 +256,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         return fields;
     }
 
-    /**
-     * Add Album
-     * <p/>
-     * <p>Only one album can be added so if one already exists it will be replaced.
-     *
-     * @param album
-     */
-    public void addAlbum(String album)
-    {
-        setAlbum(album);
-    }
-
     /**
      * Set Album
      *
@@ -281,7 +267,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         {
             throw new IllegalArgumentException(ErrorMessage.GENERAL_INVALID_NULL_ARGUMENT.getMsg());
         }
-        this.album = ID3Tags.truncate(album, this.FIELD_ALBUM_LENGTH);
+        this.album = ID3Tags.truncate(album, FIELD_ALBUM_LENGTH);
     }
 
     /**
@@ -289,7 +275,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      *
      * @return album
      */
-    public String getFirstAlbum()
+    private String getFirstAlbum()
     {
         return album;
     }
@@ -311,17 +297,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     }
 
 
-    /**
-     * Add Artist
-     * <p/>
-     * <p>Only one artist can be added so if one already exists it will be replaced.
-     *
-     * @param artist
-     */
-    public void addArtist(String artist)
-    {
-        setArtist(album);
-    }
 
 
     /**
@@ -335,7 +310,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         {
             throw new IllegalArgumentException(ErrorMessage.GENERAL_INVALID_NULL_ARGUMENT.getMsg());
         }
-        this.artist = ID3Tags.truncate(artist, this.FIELD_ARTIST_LENGTH);
+        this.artist = ID3Tags.truncate(artist, FIELD_ARTIST_LENGTH);
     }
 
     /**
@@ -343,7 +318,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      *
      * @return artist
      */
-    public String getFirstArtist()
+    private String getFirstArtist()
     {
         return artist;
     }
@@ -364,18 +339,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
-    /**
-     * Add Comment
-     * <p/>
-     * <p>Only one comment can be added so if one already exists it will be replaced.
-     *
-     * @param comment
-     */
-    public void addComment(String comment)
-    {
-        setComment(comment);
-    }
-
     /**
      * Set Comment
      *
@@ -388,7 +351,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         {
             throw new IllegalArgumentException(ErrorMessage.GENERAL_INVALID_NULL_ARGUMENT.getMsg());
         }
-        this.comment = ID3Tags.truncate(comment, this.FIELD_COMMENT_LENGTH);
+        this.comment = ID3Tags.truncate(comment, FIELD_COMMENT_LENGTH);
     }
 
     /**
@@ -417,18 +380,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         return comment;
     }
 
-    /**
-     * Add Genre
-     * <p/>
-     * <p>Only one Genre can be added so if one already exists it will be replaced.
-     *
-     * @param genre
-     */
-    public void addGenre(String genre)
-    {
-        setGenre(genre);
-    }
-
     /**
      * Sets the genreID,
      * <p/>
@@ -462,7 +413,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public String getFirstGenre()
     {
-        Integer genreId = genre & this.BYTE_TO_UNSIGNED;
+        Integer genreId = genre & BYTE_TO_UNSIGNED;
         String genreValue = GenreTypes.getInstanceOf().getValueForId(genreId);
         if (genreValue == null)
         {
@@ -483,9 +434,9 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public List<TagField> getGenre()
     {
-        if (getFirstGenre().length() > 0)
+        if (getFirst(FieldKey.GENRE).length() > 0)
         {
-            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.GENRE.name(), getFirstGenre());
+            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.GENRE.name(), getFirst(FieldKey.GENRE));
             return returnFieldToList(field);
         }
         else
@@ -494,18 +445,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
-    /**
-     * Add Title
-     * <p/>
-     * <p>Only one title can be added so if one already exists it will be replaced.
-     *
-     * @param title
-     */
-    public void addTitle(String title)
-    {
-        setTitle(title);
-    }
-
     /**
      * Set Title
      *
@@ -517,7 +456,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         {
             throw new IllegalArgumentException(ErrorMessage.GENERAL_INVALID_NULL_ARGUMENT.getMsg());
         }
-        this.title = ID3Tags.truncate(title, this.FIELD_TITLE_LENGTH);
+        this.title = ID3Tags.truncate(title, FIELD_TITLE_LENGTH);
     }
 
     /**
@@ -539,9 +478,9 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public List<TagField> getTitle()
     {
-        if (getFirstTitle().length() > 0)
+        if (getFirst(FieldKey.TITLE).length() > 0)
         {
-            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.TITLE.name(), getFirstTitle());
+            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.TITLE.name(), getFirst(FieldKey.TITLE));
             return returnFieldToList(field);
         }
         else
@@ -550,18 +489,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
-    /**
-     * Add Year
-     * <p/>
-     * <p>Only one year can be added so if one already exists it will be replaced.
-     *
-     * @param year
-     */
-    public void addYear(String year)
-    {
-        setYear(year);
-    }
-
     /**
      * Set year
      *
@@ -569,7 +496,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public void setYear(String year)
     {
-        this.year = ID3Tags.truncate(year, this.FIELD_YEAR_LENGTH);
+        this.year = ID3Tags.truncate(year, FIELD_YEAR_LENGTH);
     }
 
     /**
@@ -591,9 +518,9 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public List<TagField> getYear()
     {
-        if (getFirstYear().length() > 0)
+        if (getFirst(FieldKey.YEAR).length() > 0)
         {
-            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.YEAR.name(), getFirstYear());
+            ID3v1TagField field = new ID3v1TagField(ID3v1FieldKey.YEAR.name(), getFirst(FieldKey.YEAR));
             return returnFieldToList(field);
         }
         else
@@ -602,21 +529,11 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
-    public void addTrack(String track)
-    {
-        throw new UnsupportedOperationException("ID3v10 cannot store track numbers");
-    }
-
     public String getFirstTrack()
     {
         throw new UnsupportedOperationException("ID3v10 cannot store track numbers");
     }
 
-    public void setTrack(String track)
-    {
-        throw new UnsupportedOperationException("ID3v10 cannot store track numbers");
-    }
-
     public List<TagField> getTrack()
     {
         throw new UnsupportedOperationException("ID3v10 cannot store track numbers");
@@ -626,27 +543,27 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     {
         List<TagField> results = null;
 
-        if (TagFieldKey.ARTIST.name().equals(id))
+        if (FieldKey.ARTIST.name().equals(id))
         {
             results = getArtist();
         }
-        else if (TagFieldKey.ALBUM.name().equals(id))
+        else if (FieldKey.ALBUM.name().equals(id))
         {
             results = getAlbum();
         }
-        else if (TagFieldKey.TITLE.name().equals(id))
+        else if (FieldKey.TITLE.name().equals(id))
         {
             results = getTitle();
         }
-        else if (TagFieldKey.GENRE.name().equals(id))
+        else if (FieldKey.GENRE.name().equals(id))
         {
             results = getGenre();
         }
-        else if (TagFieldKey.YEAR.name().equals(id))
+        else if (FieldKey.YEAR.name().equals(id))
         {
             results = getYear();
         }
-        else if (TagFieldKey.COMMENT.name().equals(id))
+        else if (FieldKey.COMMENT.name().equals(id))
         {
             results = getComment();
         }
@@ -680,17 +597,24 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
 
     public boolean isEmpty()
     {
-        if (getFirstTitle().length() > 0 || getFirstArtist().length() > 0 || getFirstAlbum().length() > 0 || getFirstGenre().length() > 0 || getFirstYear().length() > 0 || getFirstComment().length() > 0)
-        {
-            return false;
-        }
-        return true;
+        return !(getFirst(FieldKey.TITLE).length() > 0 || getFirstArtist().length() > 0 || getFirstAlbum().length() > 0 || getFirst(FieldKey.GENRE).length() > 0 || getFirst(FieldKey.YEAR).length() > 0 || getFirstComment().length() > 0);
     }
 
 
-    public void set(TagField field)
+    public void setField(FieldKey genericKey, String value) throws KeyNotFoundException, FieldDataInvalidException
     {
-        TagFieldKey genericKey = TagFieldKey.valueOf(field.getId());
+        TagField tagfield = createField(genericKey,value);
+        setField(tagfield);
+    }
+
+    public void addField(FieldKey genericKey, String value) throws KeyNotFoundException, FieldDataInvalidException
+    {
+        setField(genericKey,value);
+    }
+
+    public void setField(TagField field)
+    {
+        FieldKey genericKey = FieldKey.valueOf(field.getId());
         switch (genericKey)
         {
             case ARTIST:
@@ -731,7 +655,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     /**
      * Create Tag Field using generic key
      */
-    public TagField createTagField(TagFieldKey genericKey, String value)
+    public TagField createField(FieldKey genericKey, String value)
     {
         return new ID3v1TagField(tagFieldToID3v1Field.get(genericKey).name(), value);
     }
@@ -741,9 +665,9 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         return "ISO-8859-1";
     }
 
-    public TagField getFirstField(TagFieldKey genericKey)
+    public TagField getFirstField(FieldKey genericKey)
     {
-        List<TagField> l = get(genericKey);
+        List<TagField> l = getFields(genericKey);
         return (l.size() != 0) ? l.get(0) : null;
     }
 
@@ -754,7 +678,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      * @param genericKey The generic field key
      * @return A list of {@link TagField} objects with the given &quot;id&quot;.
      */
-    public List<TagField> get(TagFieldKey genericKey)
+    public List<TagField> getFields(FieldKey genericKey)
     {
         switch (genericKey)
         {
@@ -790,7 +714,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public String getFirst(String genericKey)
     {
-        TagFieldKey matchingKey = TagFieldKey.valueOf(genericKey);
+        FieldKey matchingKey = FieldKey.valueOf(genericKey);
         if (matchingKey != null)
         {
             return getFirst(matchingKey);
@@ -808,7 +732,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      * @param genericKey
      * @return
      */
-    public String getFirst(TagFieldKey genericKey)
+    public String getFirst(FieldKey genericKey)
     {
         switch (genericKey)
         {
@@ -843,7 +767,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      *
      * @param genericKey
      */
-    public void deleteTagField(TagFieldKey genericKey)
+    public void deleteField(FieldKey genericKey)
     {
         switch (genericKey)
         {
@@ -879,20 +803,20 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public boolean equals(Object obj)
     {
-        if ((obj instanceof ID3v1Tag) == false)
+        if (!(obj instanceof ID3v1Tag))
         {
             return false;
         }
         ID3v1Tag object = (ID3v1Tag) obj;
-        if (this.album.equals(object.album) == false)
+        if (!this.album.equals(object.album))
         {
             return false;
         }
-        if (this.artist.equals(object.artist) == false)
+        if (!this.artist.equals(object.artist))
         {
             return false;
         }
-        if (this.comment.equals(object.comment) == false)
+        if (!this.comment.equals(object.comment))
         {
             return false;
         }
@@ -900,15 +824,11 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         {
             return false;
         }
-        if (this.title.equals(object.title) == false)
+        if (!this.title.equals(object.title))
         {
             return false;
         }
-        if (this.year.equals(object.year) == false)
-        {
-            return false;
-        }
-        return super.equals(obj);
+        return this.year.equals(object.year) && super.equals(obj);
     }
 
     /**
@@ -926,7 +846,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      */
     public void read(ByteBuffer byteBuffer) throws TagNotFoundException
     {
-        if (seek(byteBuffer) == false)
+        if (!seek(byteBuffer))
         {
             throw new TagNotFoundException(getLoggingFilename() + ":" + "ID3v1 tag not found");
         }
@@ -935,41 +855,41 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         byte[] dataBuffer = new byte[TAG_LENGTH];
         byteBuffer.position(0);
         byteBuffer.get(dataBuffer, 0, TAG_LENGTH);
-        title = Utils.getString(dataBuffer, FIELD_TITLE_POS, this.FIELD_TITLE_LENGTH, "ISO-8859-1").trim();
-        Matcher m = endofStringPattern.matcher(title);
-        if (m.find() == true)
+        title = Utils.getString(dataBuffer, FIELD_TITLE_POS, FIELD_TITLE_LENGTH, "ISO-8859-1").trim();
+        Matcher m = AbstractID3v1Tag.endofStringPattern.matcher(title);
+        if (m.find())
         {
             title = title.substring(0, m.start());
         }
-        artist = Utils.getString(dataBuffer, FIELD_ARTIST_POS, this.FIELD_ARTIST_LENGTH, "ISO-8859-1").trim();
-        m = endofStringPattern.matcher(artist);
-        if (m.find() == true)
+        artist = Utils.getString(dataBuffer, FIELD_ARTIST_POS, FIELD_ARTIST_LENGTH, "ISO-8859-1").trim();
+        m = AbstractID3v1Tag.endofStringPattern.matcher(artist);
+        if (m.find())
         {
             artist = artist.substring(0, m.start());
         }
-        album = Utils.getString(dataBuffer, FIELD_ALBUM_POS, this.FIELD_ALBUM_LENGTH, "ISO-8859-1").trim();
-        m = endofStringPattern.matcher(album);
+        album = Utils.getString(dataBuffer, FIELD_ALBUM_POS, FIELD_ALBUM_LENGTH, "ISO-8859-1").trim();
+        m = AbstractID3v1Tag.endofStringPattern.matcher(album);
         logger.finest(getLoggingFilename() + ":" + "Orig Album is:" + comment + ":");
-        if (m.find() == true)
+        if (m.find())
         {
             album = album.substring(0, m.start());
             logger.finest(getLoggingFilename() + ":" + "Album is:" + album + ":");
         }
-        year = Utils.getString(dataBuffer, FIELD_YEAR_POS, this.FIELD_YEAR_LENGTH, "ISO-8859-1").trim();
-        m = endofStringPattern.matcher(year);
-        if (m.find() == true)
+        year = Utils.getString(dataBuffer, FIELD_YEAR_POS, FIELD_YEAR_LENGTH, "ISO-8859-1").trim();
+        m = AbstractID3v1Tag.endofStringPattern.matcher(year);
+        if (m.find())
         {
             year = year.substring(0, m.start());
         }
-        comment = Utils.getString(dataBuffer, FIELD_COMMENT_POS, this.FIELD_COMMENT_LENGTH, "ISO-8859-1").trim();
-        m = endofStringPattern.matcher(comment);
+        comment = Utils.getString(dataBuffer, FIELD_COMMENT_POS, FIELD_COMMENT_LENGTH, "ISO-8859-1").trim();
+        m = AbstractID3v1Tag.endofStringPattern.matcher(comment);
         logger.finest(getLoggingFilename() + ":" + "Orig Comment is:" + comment + ":");
-        if (m.find() == true)
+        if (m.find())
         {
             comment = comment.substring(0, m.start());
             logger.finest(getLoggingFilename() + ":" + "Comment is:" + comment + ":");
         }
-        genre = dataBuffer[this.FIELD_GENRE_POS];
+        genre = dataBuffer[FIELD_GENRE_POS];
 
     }
 
@@ -1001,35 +921,35 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         delete(file);
         file.seek(file.length());
         //Copy the TAGID into new buffer
-        System.arraycopy(TAG_ID, this.FIELD_TAGID_POS, buffer, this.FIELD_TAGID_POS, TAG_ID.length);
-        int offset = this.FIELD_TITLE_POS;
+        System.arraycopy(TAG_ID, FIELD_TAGID_POS, buffer, FIELD_TAGID_POS, TAG_ID.length);
+        int offset = FIELD_TITLE_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveTitle())
         {
-            str = ID3Tags.truncate(title, this.FIELD_TITLE_LENGTH);
+            str = ID3Tags.truncate(title, FIELD_TITLE_LENGTH);
             for (i = 0; i < str.length(); i++)
             {
                 buffer[i + offset] = (byte) str.charAt(i);
             }
         }
-        offset = this.FIELD_ARTIST_POS;
+        offset = FIELD_ARTIST_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveArtist())
         {
-            str = ID3Tags.truncate(artist, this.FIELD_ARTIST_LENGTH);
+            str = ID3Tags.truncate(artist, FIELD_ARTIST_LENGTH);
             for (i = 0; i < str.length(); i++)
             {
                 buffer[i + offset] = (byte) str.charAt(i);
             }
         }
-        offset = this.FIELD_ALBUM_POS;
+        offset = FIELD_ALBUM_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveAlbum())
         {
-            str = ID3Tags.truncate(album, this.FIELD_ALBUM_LENGTH);
+            str = ID3Tags.truncate(album, FIELD_ALBUM_LENGTH);
             for (i = 0; i < str.length(); i++)
             {
                 buffer[i + offset] = (byte) str.charAt(i);
             }
         }
-        offset = this.FIELD_YEAR_POS;
+        offset = FIELD_YEAR_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveYear())
         {
             str = ID3Tags.truncate(year, AbstractID3v1Tag.FIELD_YEAR_LENGTH);
@@ -1038,16 +958,16 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
                 buffer[i + offset] = (byte) str.charAt(i);
             }
         }
-        offset = this.FIELD_COMMENT_POS;
+        offset = FIELD_COMMENT_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveComment())
         {
-            str = ID3Tags.truncate(comment, this.FIELD_COMMENT_LENGTH);
+            str = ID3Tags.truncate(comment, FIELD_COMMENT_LENGTH);
             for (i = 0; i < str.length(); i++)
             {
                 buffer[i + offset] = (byte) str.charAt(i);
             }
         }
-        offset = this.FIELD_GENRE_POS;
+        offset = FIELD_GENRE_POS;
         if (TagOptionSingleton.getInstance().isId3v1SaveGenre())
         {
             buffer[offset] = genre;
@@ -1082,13 +1002,27 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         return null;
     }
 
-    public TagField  createArtworkField(Artwork artwork) throws FieldDataInvalidException
+    public TagField createField(Artwork artwork) throws FieldDataInvalidException
     {
-        throw new UnsupportedOperationException("Not implemented for this format");
+        throw new UnsupportedOperationException(ErrorMessage.GENERIC_NOT_SUPPORTED.getMsg());
     }
 
-    public void createAndSetArtworkField(Artwork artwork) throws FieldDataInvalidException
+    public void setField(Artwork artwork) throws FieldDataInvalidException
     {
-        throw new UnsupportedOperationException("Not implemented for this format");
+        throw new UnsupportedOperationException(ErrorMessage.GENERIC_NOT_SUPPORTED.getMsg());
+    }
+
+    public void addField(Artwork artwork) throws FieldDataInvalidException
+    {
+        throw new UnsupportedOperationException(ErrorMessage.GENERIC_NOT_SUPPORTED.getMsg());
+    }
+    /**
+     * Delete all instance of artwork Field
+     *
+     * @throws KeyNotFoundException
+     */
+    public void deleteArtworkField() throws KeyNotFoundException
+    {
+        throw new UnsupportedOperationException(ErrorMessage.GENERIC_NOT_SUPPORTED.getMsg());
     }
 }
