@@ -37,11 +37,11 @@ import java.util.regex.Pattern;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v22Frame.java,v 1.30 2008/07/21 10:45:48 paultaylor Exp $
+ * @version $Id: ID3v22Frame.java,v 1.34 2009/11/12 15:42:57 paultaylor Exp $
  */
 public class ID3v22Frame extends AbstractID3v2Frame
 {
-    Pattern validFrameIdentifier = Pattern.compile("[A-Z][0-9A-Z]{2}");
+    private static Pattern validFrameIdentifier = Pattern.compile("[A-Z][0-9A-Z]{2}");
 
     protected static final int FRAME_ID_SIZE = 3;
     protected static final int FRAME_SIZE_SIZE = 3;
@@ -67,6 +67,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * <p/>
      * An empty body of the correct type will be automatically created. This constructor should be used when wish to
      * create a new frame from scratch using user values
+     * @param identifier
      */
     @SuppressWarnings("unchecked")
     public ID3v22Frame(String identifier)
@@ -133,6 +134,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * Copy Constructor
      * <p/>
      * Creates a new v22 frame based on another v22 frame
+     * @param frame
      */
     public ID3v22Frame(ID3v22Frame frame)
     {
@@ -147,17 +149,15 @@ public class ID3v22Frame extends AbstractID3v2Frame
         {
             logger.info("V2:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
             this.frameBody = (AbstractID3v2FrameBody) ID3Tags.copyObject(frame.getBody());
-            return;
         }
         // Is it a known v3 frame which needs forcing to v2 frame e.g. APIC - PIC
-        else if (ID3Tags.isID3v23FrameIdentifier(frame.getIdentifier()) == true)
+        else if (ID3Tags.isID3v23FrameIdentifier(frame.getIdentifier()))
         {
             identifier = ID3Tags.forceFrameID23To22(frame.getIdentifier());
             if (identifier != null)
             {
                 logger.info("V2:Force:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
                 this.frameBody = this.readBody(identifier, (AbstractID3v2FrameBody) frame.getBody());
-                return;
             }
             // No mechanism exists to convert it to a v22 frame
             else
@@ -181,7 +181,6 @@ public class ID3v22Frame extends AbstractID3v2Frame
                 this.frameBody = new FrameBodyDeprecated((FrameBodyDeprecated) frame.getBody());
                 identifier = frame.getIdentifier();
                 logger.info("DEPRECATED:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
-                return;
             }
         }
         // Unknown Frame e.g NCON
@@ -190,7 +189,6 @@ public class ID3v22Frame extends AbstractID3v2Frame
             this.frameBody = new FrameBodyUnsupported((FrameBodyUnsupported) frame.getBody());
             identifier = frame.getIdentifier();
             logger.info("v2:UNKNOWN:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
-            return;
         }
     }
 
@@ -198,11 +196,12 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * Creates a new ID3v22 Frame from another frame of a different tag version
      *
      * @param frame to construct the new frame from
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v22Frame(AbstractID3v2Frame frame) throws InvalidFrameException
     {
         logger.info("Creating frame from a frame of a different version");
-        if ((frame instanceof ID3v22Frame == true) && (frame instanceof ID3v23Frame == false))
+        if (frame instanceof ID3v22Frame)
         {
             throw new UnsupportedOperationException("Copy Constructor not called. Please type cast the argument");
         }
@@ -226,6 +225,8 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * Creates a new ID3v22Frame datatype by reading from byteBuffer.
      *
      * @param byteBuffer to read from
+     * @param loggingFilename
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v22Frame(ByteBuffer byteBuffer, String loggingFilename) throws InvalidFrameException
     {
@@ -238,6 +239,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      *
      * @param byteBuffer to read from
      * @deprecated use {@link #ID3v22Frame(ByteBuffer,String)} instead
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v22Frame(ByteBuffer byteBuffer) throws InvalidFrameException
     {
@@ -276,7 +278,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
         logger.info("Read Frame from file identifier is:" + identifier);
 
         // Is this a valid identifier?
-        if (isValidID3v2FrameIdentifier(identifier) == false)
+        if (!isValidID3v2FrameIdentifier(identifier))
         {
             logger.info("Invalid identifier:" + identifier);
             byteBuffer.position(byteBuffer.position() - (FRAME_ID_SIZE - 1));
@@ -313,7 +315,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
                 {
                     // Is it a valid v22 identifier so should be able to find a
                     // frame body for it.
-                    if (ID3Tags.isID3v22FrameIdentifier(identifier) == true)
+                    if (ID3Tags.isID3v22FrameIdentifier(identifier))
                     {
                         id = identifier;
                     }
@@ -344,6 +346,8 @@ public class ID3v22Frame extends AbstractID3v2Frame
 
     /**
      * Read Frame Size, which has to be decoded
+     * @param buffer
+     * @return
      */
     private int decodeSize(byte[] buffer)
     {
@@ -396,6 +400,8 @@ public class ID3v22Frame extends AbstractID3v2Frame
     /**
      * Write Frame Size (can now be accurately calculated, have to convert 4 byte int
      * to 3 byte format.
+     * @param headerBuffer
+     * @param size
      */
     private void encodeSize(ByteBuffer headerBuffer, int size)
     {
@@ -414,7 +420,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      */
     public boolean isValidID3v2FrameIdentifier(String identifier)
     {
-        Matcher m = validFrameIdentifier.matcher(identifier);
+        Matcher m = ID3v22Frame.validFrameIdentifier.matcher(identifier);
         return m.matches();
     }
 
