@@ -39,11 +39,11 @@ import java.util.zip.Inflater;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v23Frame.java,v 1.38 2008/11/12 16:41:39 paultaylor Exp $
+ * @version $Id: ID3v23Frame.java,v 1.46 2009/11/12 15:42:57 paultaylor Exp $
  */
 public class ID3v23Frame extends AbstractID3v2Frame
 {
-    Pattern validFrameIdentifier = Pattern.compile("[A-Z][0-9A-Z]{3}");
+    private static Pattern validFrameIdentifier = Pattern.compile("[A-Z][0-9A-Z]{3}");
 
     protected static final int FRAME_ID_SIZE = 4;
     protected static final int FRAME_FLAGS_SIZE = 2;
@@ -67,6 +67,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
      * <p>An empty body of the correct type will be automatically created.
      * This constructor should be used when wish to create a new
      * frame from scratch using user data.
+     * @param identifier
      */
     public ID3v23Frame(String identifier)
     {
@@ -79,6 +80,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
      * Copy Constructor
      * <p/>
      * Creates a new v23 frame  based on another v23 frame
+     * @param frame
      */
     public ID3v23Frame(ID3v23Frame frame)
     {
@@ -91,11 +93,12 @@ public class ID3v23Frame extends AbstractID3v2Frame
      * Creates a new ID3v23Frame  based on another frame.
      *
      * @param frame
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v23Frame(AbstractID3v2Frame frame) throws InvalidFrameException
     {
-        logger.info("Creating frame from a frame of a different version");
-        if ((frame instanceof ID3v23Frame) && (frame instanceof ID3v24Frame == false))
+        logger.finer("Creating frame from a frame of a different version");
+        if (frame instanceof ID3v23Frame)
         {
             throw new UnsupportedOperationException("Copy Constructor not called. Please type cast the argument");
         }
@@ -107,46 +110,8 @@ public class ID3v23Frame extends AbstractID3v2Frame
 
         if (frame instanceof ID3v24Frame)
         {
-            if (ID3Tags.isID3v24FrameIdentifier(frame.getIdentifier()))
-            {
-                //Version between v4 and v3
-                identifier = ID3Tags.convertFrameID24To23(frame.getIdentifier());
-                if (identifier != null)
-                {
-                    logger.info("V4:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
-                    this.frameBody = (AbstractTagFrameBody) ID3Tags.copyObject(frame.getBody());
-                    this.frameBody.setHeader(this);
-                    return;
-                }
-                else
-                {
-                    //Is it a known v4 frame which needs forcing to v3 frame e.g. TDRC - TYER,TDAT
-                    identifier = ID3Tags.forceFrameID24To23(frame.getIdentifier());
-                    if (identifier != null)
-                    {
-                        logger.info("V4:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
-                        this.frameBody = this.readBody(identifier, (AbstractID3v2FrameBody) frame.getBody());
-                        this.frameBody.setHeader(this);
-                        return;
-                    }
-                    //It is a v24 frame that is not known and cannot be forced in v23 e.g TDRL,in which case
-                    //we convert to a framebody unsupported by writing contents as a byte array and feeding
-                    //it into FrameBodyUnsupported
-                    else
-                    {
-                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
-                        ((AbstractID3v2FrameBody) frame.getBody()).write(baos);
-
-                        identifier = frame.getIdentifier();
-                        this.frameBody = new FrameBodyUnsupported(identifier, baos.toByteArray());
-                        this.frameBody.setHeader(this);
-                        logger.info("V4:Orig id is:" + frame.getIdentifier() + ":New Id Unsupported is:" + identifier);
-                        return;
-                    }
-                }
-            }
-            //Unknown Frame e.g NCON
-            else if (frame.getBody() instanceof FrameBodyUnsupported)
+            //Unknown Frame e.g NCON,also protects when known id but has unsupported frame body
+            if (frame.getBody() instanceof FrameBodyUnsupported)
             {
                 this.frameBody = new FrameBodyUnsupported((FrameBodyUnsupported) frame.getBody());
                 this.frameBody.setHeader(this);
@@ -175,6 +140,45 @@ public class ID3v23Frame extends AbstractID3v2Frame
                     return;
                 }
             }
+            else if (ID3Tags.isID3v24FrameIdentifier(frame.getIdentifier()))
+            {
+                logger.finer("isID3v24FrameIdentifier");
+                //Version between v4 and v3
+                identifier = ID3Tags.convertFrameID24To23(frame.getIdentifier());
+                if (identifier != null)
+                {
+                    logger.finer("V4:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
+                    this.frameBody = (AbstractTagFrameBody) ID3Tags.copyObject(frame.getBody());
+                    this.frameBody.setHeader(this);
+                    return;
+                }
+                else
+                {
+                    //Is it a known v4 frame which needs forcing to v3 frame e.g. TDRC - TYER,TDAT
+                    identifier = ID3Tags.forceFrameID24To23(frame.getIdentifier());
+                    if (identifier != null)
+                    {
+                        logger.finer("V4:Orig id is:" + frame.getIdentifier() + ":New id is:" + identifier);
+                        this.frameBody = this.readBody(identifier, (AbstractID3v2FrameBody) frame.getBody());
+                        this.frameBody.setHeader(this);
+                        return;
+                    }
+                    //It is a v24 frame that is not known and cannot be forced in v23 e.g TDRL,in which case
+                    //we convert to a framebody unsupported by writing contents as a byte array and feeding
+                    //it into FrameBodyUnsupported
+                    else
+                    {
+                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                        ((AbstractID3v2FrameBody) frame.getBody()).write(baos);
+
+                        identifier = frame.getIdentifier();
+                        this.frameBody = new FrameBodyUnsupported(identifier, baos.toByteArray());
+                        this.frameBody.setHeader(this);
+                        logger.finer("V4:Orig id is:" + frame.getIdentifier() + ":New Id Unsupported is:" + identifier);
+                        return;
+                    }
+                }
+            }
             // Unable to find a suitable framebody, this shold not happen
             else
             {
@@ -228,13 +232,15 @@ public class ID3v23Frame extends AbstractID3v2Frame
             }
         }
 
-        logger.warning("Frame is unknown version");
+        logger.warning("Frame is unknown version:"+frame.getClass());
     }
 
     /**
      * Creates a new ID3v23Frame datatype by reading from byteBuffer.
      *
      * @param byteBuffer to read from
+     * @param loggingFilename
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v23Frame(ByteBuffer byteBuffer, String loggingFilename) throws InvalidFrameException
     {
@@ -247,6 +253,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
      *
      * @param byteBuffer to read from
      * @deprecated use {@link #ID3v23Frame(ByteBuffer,String)} instead
+     * @throws org.jaudiotagger.tag.InvalidFrameException
      */
     public ID3v23Frame(ByteBuffer byteBuffer) throws InvalidFrameException
     {
@@ -275,7 +282,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
      */
     public boolean equals(Object obj)
     {
-        if ((obj instanceof ID3v23Frame) == false)
+        if (!(obj instanceof ID3v23Frame))
         {
             return false;
         }
@@ -284,11 +291,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
         {
             return false;
         }
-        if (this.encodingFlags.getFlags() != object.encodingFlags.getFlags())
-        {
-            return false;
-        }
-        return super.equals(obj);
+        return this.encodingFlags.getFlags() == object.encodingFlags.getFlags() && super.equals(obj);
     }
 
     /**
@@ -313,7 +316,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
         identifier = new String(buffer);
 
         // Is this a valid identifier?
-        if (isValidID3v2FrameIdentifier(identifier) == false)
+        if (!isValidID3v2FrameIdentifier(identifier))
         {
             logger.info(getLoggingFilename() + ":Invalid identifier:" + identifier);
             byteBuffer.position(byteBuffer.position() - (FRAME_ID_SIZE - 1));
@@ -337,7 +340,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
         }
         else if (frameSize > byteBuffer.remaining())
         {
-            logger.warning(getLoggingFilename() + ":Invalid Frame size larger than size before mp3 audio:" + identifier);
+            logger.warning(getLoggingFilename() + ":Invalid Frame size of " +frameSize +" larger than size of" + byteBuffer.remaining() + " before mp3 audio:" + identifier);
             throw new InvalidFrameException(identifier + " is invalid frame");
         }
 
@@ -354,7 +357,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
         {
             // It is a valid v23 identifier so should be able to find a
             //  frame body for it.
-            if (ID3Tags.isID3v23FrameIdentifier(identifier) == true)
+            if (ID3Tags.isID3v23FrameIdentifier(identifier))
             {
                 id = identifier;
             }
@@ -560,6 +563,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
 
         /**
          * Use this constructor when convert a v24 frame
+         * @param statusFlags
          */
         StatusFlags(ID3v24Frame.StatusFlags statusFlags)
         {
@@ -585,7 +589,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
         protected void modifyFlags()
         {
             String str = getIdentifier();
-            if (ID3v23Frames.getInstanceOf().isDiscardIfFileAltered(str) == true)
+            if (ID3v23Frames.getInstanceOf().isDiscardIfFileAltered(str))
             {
                 writeFlags |= (byte) MASK_FILE_ALTER_PRESERVATION;
                 writeFlags &= (byte) ~MASK_TAG_ALTER_PRESERVATION;
@@ -695,7 +699,7 @@ public class ID3v23Frame extends AbstractID3v2Frame
      */
     public boolean isValidID3v2FrameIdentifier(String identifier)
     {
-        Matcher m = validFrameIdentifier.matcher(identifier);
+        Matcher m = ID3v23Frame.validFrameIdentifier.matcher(identifier);
         return m.matches();
     }
 
