@@ -10,32 +10,28 @@
  *  See the GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License along with this library; if not,
- *  you can get a copy from http://www.opensource.org/licenses/lgpl-license.php or write to the Free Software
+ *  you can getFields a copy from http://www.opensource.org/licenses/lgpl-license.php or write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 package org.jaudiotagger.tag.id3;
 
 import org.jaudiotagger.FileConstants;
-import org.jaudiotagger.logging.FileSystemMessage;
-import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.audio.mp3.MP3File;
-import org.jaudiotagger.audio.exceptions.UnableToCreateFileException;
-import org.jaudiotagger.audio.exceptions.UnableToModifyFileException;
+import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.tag.*;
-import org.jaudiotagger.tag.reference.PictureTypes;
 import org.jaudiotagger.tag.datatype.Artwork;
 import org.jaudiotagger.tag.datatype.DataTypes;
 import org.jaudiotagger.tag.id3.framebody.*;
+import org.jaudiotagger.tag.reference.PictureTypes;
 
 import java.io.File;
 import java.io.IOException;
-import java.io.RandomAccessFile;
-import java.io.FileNotFoundException;
 import java.nio.ByteBuffer;
-import java.nio.channels.FileChannel;
-import java.nio.channels.FileLock;
 import java.nio.channels.WritableByteChannel;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Comparator;
+import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.logging.Level;
 
 /**
@@ -43,7 +39,7 @@ import java.util.logging.Level;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v23Tag.java,v 1.44 2008/12/10 13:14:27 paultaylor Exp $
+ * @version $Id: ID3v23Tag.java,v 1.52 2009/11/12 15:42:58 paultaylor Exp $
  */
 public class ID3v23Tag extends AbstractID3v2Tag
 {
@@ -101,14 +97,14 @@ public class ID3v23Tag extends AbstractID3v2Tag
     protected boolean extended = false;
 
     /**
-     * CRC Checksum
+     * Crcdata Checksum in extended header
      */
-    protected int crcData = 0;
+    private int crc32;
 
     /**
      * Tag padding
      */
-    protected int paddingSize = 0;
+    private int paddingSize = 0;
 
     /**
      * All frames in the tag uses unsynchronisation
@@ -125,6 +121,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
     public static final byte MAJOR_VERSION = 3;
     public static final byte REVISION = 0;
 
+
     /**
      * Retrieve the Release
      */
@@ -150,6 +147,14 @@ public class ID3v23Tag extends AbstractID3v2Tag
     }
 
 
+    /**
+     * @return  Cyclic Redundancy Check 32 Value
+     */
+    public int getCrc32()
+    {
+        return crc32;
+    }
+
     /**
      * Creates a new empty ID3v2_3 datatype.
      */
@@ -172,92 +177,40 @@ public class ID3v23Tag extends AbstractID3v2Tag
             this.crcDataFlag = copyObject.crcDataFlag;
             this.experimental = copyObject.experimental;
             this.extended = copyObject.extended;
-            this.crcData = copyObject.crcData;
+            this.crc32 = copyObject.crc32;
             this.paddingSize = copyObject.paddingSize;
         }
     }
 
-    /**
-     * Copy frames from one tag into a v2.3 tag
-     *
-     * @param copyObject
-     */
-    protected void copyFrames(AbstractID3v2Tag copyObject)
+
+
+    protected void addFrame(AbstractID3v2Frame frame)
     {
-        logger.info("Copying Frames,there are:" + copyObject.frameMap.keySet().size() + " different types");
-        frameMap = new LinkedHashMap();
-
-        //Copy Frames that are a valid 2.3 type
-        Iterator<String> iterator = copyObject.frameMap.keySet().iterator();
-        AbstractID3v2Frame frame;
-        ID3v23Frame newFrame = null;
-        while (iterator.hasNext())
+        try
         {
-            String id = iterator.next();
-            Object o = copyObject.frameMap.get(id);
-            if (o instanceof AbstractID3v2Frame)
+            //Special case to handle TDRC frame from V24 that needs breaking up into seperate frame in V23
+            if ((frame.getIdentifier().equals(ID3v24Frames.FRAME_ID_YEAR)) && (frame.getBody() instanceof FrameBodyTDRC))
             {
-                frame = (AbstractID3v2Frame) o;
-                logger.info("Frame is:" + frame.getIdentifier());
-
-                //Special case v24 tdrc may need converting to multiple frames, only convert when
-                //it is a valid tdrc to cope with tdrc being added illegally to a v23 tag which is then
-                //converted to v24 tag and back again.
-                if ((frame.getIdentifier().equals(ID3v24Frames.FRAME_ID_YEAR)) && (frame.getBody() instanceof FrameBodyTDRC))
-                {
-                    translateFrame(frame);
-                }
-                //Usual Case
-                else
-                {
-                    try
-                    {
-
-                        newFrame = new ID3v23Frame(frame);
-                        logger.info("Adding Frame:" + newFrame.getIdentifier());
-                        frameMap.put(newFrame.getIdentifier(), newFrame);
-                    }
-                    catch (InvalidFrameException ife)
-                    {
-                        logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier());
-                    }
-                }
+                translateFrame(frame);
             }
-            //Multi Frames
-            else if (o instanceof ArrayList)
+            else
             {
-                ArrayList<AbstractID3v2Frame> multiFrame = new ArrayList<AbstractID3v2Frame>();
-                for (ListIterator<AbstractID3v2Frame> li = ((ArrayList<AbstractID3v2Frame>) o).listIterator(); li.hasNext();)
-                {
-                    frame = li.next();
-                    logger.info("Frame is MultiFrame:" + frame.getIdentifier());
-                    try
-                    {
-                        newFrame = new ID3v23Frame(frame);
-                        multiFrame.add(newFrame);
-                    }
-                    catch (InvalidFrameException ife)
-                    {
-                        logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier(), ife);
-                    }
-                }
-                //Ensure that the list actually contains at lest one value before adding
-                if(multiFrame.size()>0)
-                {
-                    if (newFrame != null)
-                    {
-                        logger.info("Adding MultiFrame:" + newFrame.getIdentifier());
-                        frameMap.put(newFrame.getIdentifier(), multiFrame);
-                    }
-                }
+                ID3v23Frame newFrame = new ID3v23Frame(frame);
+                copyFrameIntoMap(newFrame.getIdentifier(), newFrame);
             }
         }
+        catch (InvalidFrameException ife)
+        {
+            logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier());
+        }
     }
 
     /**
      * This is used when we need to translate a single frame into multiple frames,
      * currently required for v24 TDRC frames.
+     * @param frame
      */
+    //TODO will overwrite any existing TYER or TIME frame, do we ever want multiples of these
     protected void translateFrame(AbstractID3v2Frame frame)
     {
         FrameBodyTDRC tmpBody = (FrameBodyTDRC) frame.getBody();
@@ -287,6 +240,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
 
     /**
      * Copy Constructor, creates a new ID3v2_3 Tag based on another ID3v2_3 Tag
+     * @param copyObject
      */
     public ID3v23Tag(ID3v23Tag copyObject)
     {
@@ -300,6 +254,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
 
     /**
      * Constructs a new tag based upon another tag of different version/type
+     * @param mp3tag
      */
     public ID3v23Tag(AbstractTag mp3tag)
     {
@@ -310,7 +265,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
         {
             ID3v24Tag convertedTag;
             //Should use simpler copy constructor
-            if ((mp3tag instanceof ID3v24Tag == false) && (mp3tag instanceof ID3v23Tag == true))
+            if (mp3tag instanceof ID3v23Tag)
             {
                 throw new UnsupportedOperationException("Copy Constructor not called. Please type cast the argument");
             }
@@ -378,10 +333,10 @@ public class ID3v23Tag extends AbstractID3v2Tag
         int size = TAG_HEADER_LENGTH;
         if (extended)
         {
-            size += this.TAG_EXT_HEADER_LENGTH;
+            size += TAG_EXT_HEADER_LENGTH;
             if (crcDataFlag)
             {
-                size += this.TAG_EXT_HEADER_CRC_LENGTH;
+                size += TAG_EXT_HEADER_CRC_LENGTH;
             }
         }
         size += super.getSize();
@@ -396,12 +351,12 @@ public class ID3v23Tag extends AbstractID3v2Tag
      */
     public boolean equals(Object obj)
     {
-        if ((obj instanceof ID3v23Tag) == false)
+        if (!(obj instanceof ID3v23Tag))
         {
             return false;
         }
         ID3v23Tag object = (ID3v23Tag) obj;
-        if (this.crcData != object.crcData)
+        if (this.crc32 != object.crc32)
         {
             return false;
         }
@@ -417,11 +372,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
         {
             return false;
         }
-        if (this.paddingSize != object.paddingSize)
-        {
-            return false;
-        }
-        return super.equals(obj);
+        return this.paddingSize == object.paddingSize && super.equals(obj);
     }
 
     /**
@@ -440,91 +391,154 @@ public class ID3v23Tag extends AbstractID3v2Tag
         // Read the size, this is size of tag not including  the tag header
         int size = ID3SyncSafeInteger.bufferToValue(buffer);
 
-        //Return the exact size of tag as set in the tag header
+        //Return the exact size of tag as setField in the tag header
         return size + TAG_HEADER_LENGTH;
     }
 
     /**
-     * Read tag from File
+     * Read header flags
      *
-     * @param buffer The buffer to read the ID3v23 Tag from
+     * <p>Log info messages for falgs that have been set and log warnings when bits have been set for unknown flags</p>
+     * @param buffer
+     * @throws TagException
      */
-    public void read(ByteBuffer buffer) throws TagException
+    private void readHeaderFlags(ByteBuffer buffer) throws TagException
     {
-        int size;
-        if (seek(buffer) == false)
-        {
-            throw new TagNotFoundException(getIdentifier() + " tag not found");
-        }
-        logger.info(getLoggingFilename() + ":" + "Reading ID3v23 tag");
-
-        //Flags
+        //Allowable Flags
         byte flags = buffer.get();
         unsynchronization = (flags & MASK_V23_UNSYNCHRONIZATION) != 0;
         extended = (flags & MASK_V23_EXTENDED_HEADER) != 0;
         experimental = (flags & MASK_V23_EXPERIMENTAL) != 0;
 
+        //Not allowable/Unknown Flags
+        if ((flags & FileConstants.BIT4) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT4));
+        }
+
+        if ((flags & FileConstants.BIT3) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT3));
+        }
+
+        if ((flags & FileConstants.BIT2) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT2));
+        }
+
+        if ((flags & FileConstants.BIT1) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT1));
+        }
+
+        if ((flags & FileConstants.BIT0) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT0));
+        }
+
         if (isUnsynchronization())
         {
-            logger.info(getLoggingFilename() + ":" + "ID3v23 Tag is unsynchronized");
+            logger.info(ErrorMessage.ID3_TAG_UNSYNCHRONIZED.getMsg(getLoggingFilename()));
         }
 
         if (extended)
         {
-            logger.warning(getLoggingFilename() + ":" + "ID3v23 Tag is extended");
+            logger.info(ErrorMessage.ID3_TAG_EXTENDED.getMsg(getLoggingFilename()));
         }
 
         if (experimental)
         {
-            logger.warning(getLoggingFilename() + ":" + "ID3v23 Tag is experimental");
+            logger.info(ErrorMessage.ID3_TAG_EXPERIMENTAL.getMsg(getLoggingFilename()));
         }
+    }
 
-        // Read the size, this is size of tag not including  the tag header
+    /**
+     * Read the optional extended header
+     *
+     * @param buffer
+     * @param size
+     */
+    private void readExtendedHeader(ByteBuffer buffer, int size)
+    {
+        // Int is 4 bytes.
+        int extendedHeaderSize = buffer.getInt();
+        // Extended header without CRC Data
+        if (extendedHeaderSize == TAG_EXT_HEADER_DATA_LENGTH)
+        {
+            //Flag should not be setField , if is log a warning
+            byte extFlag = buffer.get();
+            crcDataFlag = (extFlag & MASK_V23_CRC_DATA_PRESENT) != 0;
+            if (crcDataFlag)
+            {
+                logger.warning(ErrorMessage.ID3_TAG_CRC_FLAG_SET_INCORRECTLY.getMsg(getLoggingFilename()));
+            }
+            //2nd Flag Byte (not used)
+            buffer.get();
+
+            //Take padding and ext header size off the size to be read
+            paddingSize=buffer.getInt();
+            if(paddingSize>0)
+            {
+                logger.info(ErrorMessage.ID3_TAG_PADDING_SIZE.getMsg(getLoggingFilename(),paddingSize));
+            }
+            size = size - ( paddingSize + TAG_EXT_HEADER_LENGTH);
+        }
+        else if (extendedHeaderSize == TAG_EXT_HEADER_DATA_LENGTH + TAG_EXT_HEADER_CRC_LENGTH)
+        {
+            logger.info(ErrorMessage.ID3_TAG_CRC.getMsg(getLoggingFilename()));
+
+            //Flag should be setField, if nor just act as if it is
+            byte extFlag = buffer.get();
+            crcDataFlag = (extFlag & MASK_V23_CRC_DATA_PRESENT) != 0;
+            if (!crcDataFlag)
+            {
+                logger.warning(ErrorMessage.ID3_TAG_CRC_FLAG_SET_INCORRECTLY.getMsg(getLoggingFilename()));
+            }
+            //2nd Flag Byte (not used)
+            buffer.get();
+            //Take padding size of size to be read
+            paddingSize = buffer.getInt();
+            if(paddingSize>0)
+            {
+                logger.info(ErrorMessage.ID3_TAG_PADDING_SIZE.getMsg(getLoggingFilename(),paddingSize));
+            }
+            size = size - (paddingSize + TAG_EXT_HEADER_LENGTH + TAG_EXT_HEADER_CRC_LENGTH);
+            //CRC Data
+            crc32 = buffer.getInt();
+            logger.info(ErrorMessage.ID3_TAG_CRC_SIZE.getMsg(getLoggingFilename(),crc32));
+        }
+        //Extended header size is only allowed to be six or ten bytes so this is invalid but instead
+        //of giving up lets guess its six bytes and carry on and see if we can read file ok
+        else
+        {
+            logger.warning(ErrorMessage.ID3_EXTENDED_HEADER_SIZE_INVALID.getMsg(getLoggingFilename(), extendedHeaderSize));
+            buffer.position(buffer.position() - FIELD_TAG_EXT_SIZE_LENGTH);
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void read(ByteBuffer buffer) throws TagException
+    {
+        int size;
+        if (!seek(buffer))
+        {
+            throw new TagNotFoundException(getIdentifier() + " tag not found");
+        }
+        logger.info(getLoggingFilename() + ":" + "Reading ID3v23 tag");
+
+        readHeaderFlags(buffer);
+
+        // Read the size, this is size of tag not including the tag header
         size = ID3SyncSafeInteger.bufferToValue(buffer);
-        logger.info(getLoggingFilename() + ":Tag size is:" + size + " according to header (does not include header size, add 10)");
+        logger.info(ErrorMessage.ID_TAG_SIZE.getMsg(getLoggingFilename(),size));
 
         //Extended Header
         if (extended)
         {
-            // Int is 4 bytes.
-            int extendedHeaderSize = buffer.getInt();
-            // Extended header without CRC Data
-            if (extendedHeaderSize == TAG_EXT_HEADER_DATA_LENGTH)
-            {
-                //Flag
-                byte extFlag = buffer.get();
-                crcDataFlag = (extFlag & MASK_V23_CRC_DATA_PRESENT) != 0;
-                if (crcDataFlag == true)
-                {
-                    throw new InvalidTagException(getLoggingFilename() + ":CRC Data flag not set correctly.");
-                }
-                //Flag Byte (not used)
-                buffer.get();
-                //Take padding and ext header size off size to be read
-                size = size - (buffer.getInt() + TAG_EXT_HEADER_LENGTH);
-            }
-            else if (extendedHeaderSize == TAG_EXT_HEADER_DATA_LENGTH + TAG_EXT_HEADER_CRC_LENGTH)
-            {
-                //Flag
-                byte extFlag = buffer.get();
-                crcDataFlag = (extFlag & MASK_V23_CRC_DATA_PRESENT) != 0;
-                if (crcDataFlag == false)
-                {
-                    throw new InvalidTagException(getLoggingFilename() + ":CRC Data flag not set correctly.");
-                }
-                //Flag Byte (not used)
-                buffer.get();
-                //Take padding size of size to be read
-                size = size - (buffer.getInt() + TAG_EXT_HEADER_LENGTH + TAG_EXT_HEADER_CRC_LENGTH);
-                //CRC Data
-                crcData = buffer.getInt();
-            }
-            else
-            {
-                throw new InvalidTagException("Invalid Extended Header Size.");
-            }
-            logger.info(getLoggingFilename() + ":has Extended Header so adjusted Tag size is:" + size);
-
+            readExtendedHeader(buffer, size);
         }
 
         //Slice Buffer, so position markers tally with size (i.e do not include tagheader)
@@ -594,8 +608,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
                 //Dont try and find any more frames
                 break;
             }
-            ;
-        }
+            }
     }
 
     /**
@@ -613,7 +626,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
     {
         // Flags,currently we never calculate the CRC
         // and if we dont calculate them cant keep orig values. Tags are not
-        // experimental and we never create extended header to keep things simple.
+        // experimental and we never createField extended header to keep things simple.
         extended = false;
         experimental = false;
         crcDataFlag = false;
@@ -651,10 +664,10 @@ public class ID3v23Tag extends AbstractID3v2Tag
         int additionalHeaderSize = 0;
         if (extended)
         {
-            additionalHeaderSize += this.TAG_EXT_HEADER_LENGTH;
+            additionalHeaderSize += TAG_EXT_HEADER_LENGTH;
             if (crcDataFlag)
             {
-                additionalHeaderSize += this.TAG_EXT_HEADER_CRC_LENGTH;
+                additionalHeaderSize += TAG_EXT_HEADER_CRC_LENGTH;
             }
         }
 
@@ -668,14 +681,14 @@ public class ID3v23Tag extends AbstractID3v2Tag
             byte extFlagsByte2 = 0;
 
             //Contains CRCData
-            if (crcDataFlag == true)
+            if (crcDataFlag)
             {
                 headerBuffer.putInt(TAG_EXT_HEADER_DATA_LENGTH + TAG_EXT_HEADER_CRC_LENGTH);
                 extFlagsByte1 |= MASK_V23_CRC_DATA_PRESENT;
                 headerBuffer.put(extFlagsByte1);
                 headerBuffer.put(extFlagsByte2);
                 headerBuffer.putInt(paddingSize);
-                headerBuffer.putInt(crcData);
+                headerBuffer.putInt(crc32);
             }
             //Just extended Header
             else
@@ -711,14 +724,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
         logger.info(getLoggingFilename() + ":bodybytebuffer:sizebeforeunsynchronisation:" + bodyByteBuffer.length);
 
         // Unsynchronize if option enabled and unsync required
-        if (TagOptionSingleton.getInstance().isUnsyncTags())
-        {
-            unsynchronization = ID3Unsynchronization.requiresUnsynchronization(bodyByteBuffer);
-        }
-        else
-        {
-            unsynchronization = false;
-        }
+        unsynchronization = TagOptionSingleton.getInstance().isUnsyncTags() && ID3Unsynchronization.requiresUnsynchronization(bodyByteBuffer);
         if (isUnsynchronization())
         {
             bodyByteBuffer = ID3Unsynchronization.unsynchronize(bodyByteBuffer);
@@ -732,15 +738,13 @@ public class ID3v23Tag extends AbstractID3v2Tag
         logger.info(getLoggingFilename() + ":Padding:" + padding);
 
         ByteBuffer headerBuffer = writeHeaderToBuffer(padding, bodyByteBuffer.length);
-        writeBufferToFile(file,headerBuffer, bodyByteBuffer,padding,sizeIncPadding,audioStartLocation);
+        writeBufferToFile(file, headerBuffer, bodyByteBuffer, padding, sizeIncPadding, audioStartLocation);
     }
 
     /**
-     * Write tag to channel
-     *
-     * @param channel
-     * @throws IOException
+     * {@inheritDoc}
      */
+    @Override
     public void write(WritableByteChannel channel) throws IOException
     {
         logger.info(getLoggingFilename() + ":Writing tag to channel");
@@ -749,14 +753,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
         logger.info(getLoggingFilename() + ":bodybytebuffer:sizebeforeunsynchronisation:" + bodyByteBuffer.length);
 
         // Unsynchronize if option enabled and unsync required
-        if (TagOptionSingleton.getInstance().isUnsyncTags())
-        {
-            unsynchronization = ID3Unsynchronization.requiresUnsynchronization(bodyByteBuffer);
-        }
-        else
-        {
-            unsynchronization = false;
-        }
+        unsynchronization = TagOptionSingleton.getInstance().isUnsyncTags() && ID3Unsynchronization.requiresUnsynchronization(bodyByteBuffer);
         if (isUnsynchronization())
         {
             bodyByteBuffer = ID3Unsynchronization.unsynchronize(bodyByteBuffer);
@@ -783,7 +780,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
         MP3File.getStructureFormatter().addElement(TYPE_UNSYNCHRONISATION, this.isUnsynchronization());
         MP3File.getStructureFormatter().addElement(TYPE_EXTENDED, this.extended);
         MP3File.getStructureFormatter().addElement(TYPE_EXPERIMENTAL, this.experimental);
-        MP3File.getStructureFormatter().addElement(TYPE_CRCDATA, this.crcData);
+        MP3File.getStructureFormatter().addElement(TYPE_CRCDATA, this.crc32);
         MP3File.getStructureFormatter().addElement(TYPE_PADDINGSIZE, this.paddingSize);
         MP3File.getStructureFormatter().closeHeadingElement(TYPE_HEADER);
         //Body
@@ -866,6 +863,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
      *
      * @param id3v23FieldKey
      * @return
+     * @throws org.jaudiotagger.tag.KeyNotFoundException
      */
     public String getFirst(ID3v23FieldKey id3v23FieldKey) throws KeyNotFoundException
     {
@@ -873,13 +871,39 @@ public class ID3v23Tag extends AbstractID3v2Tag
         {
             throw new KeyNotFoundException();
         }
-        return super.doGetFirst(new FrameAndSubId(id3v23FieldKey.getFrameId(), id3v23FieldKey.getSubId()));
+
+        FrameAndSubId frameAndSubId = new FrameAndSubId(id3v23FieldKey.getFrameId(), id3v23FieldKey.getSubId());
+        if (id3v23FieldKey == ID3v23FieldKey.TRACK)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTRCK)frame.getBody()).getTrackNo());
+        }
+        else if (id3v23FieldKey == ID3v23FieldKey.TRACK_TOTAL)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTRCK)frame.getBody()).getTrackTotal());
+        }
+        else if (id3v23FieldKey == ID3v23FieldKey.DISC_NO)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTPOS)frame.getBody()).getDiscNo());
+        }
+        else if (id3v23FieldKey == ID3v23FieldKey.DISC_TOTAL)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTPOS)frame.getBody()).getDiscTotal());
+        }
+        else
+        {
+            return super.doGetFirst(frameAndSubId);
+        }
     }
 
     /**
      * Delete fields with this id3v23FieldKey
      *
      * @param id3v23FieldKey
+     * @throws org.jaudiotagger.tag.KeyNotFoundException
      */
     public void deleteTagField(ID3v23FieldKey id3v23FieldKey) throws KeyNotFoundException
     {
@@ -891,7 +915,7 @@ public class ID3v23Tag extends AbstractID3v2Tag
     }
 
 
-    protected FrameAndSubId getFrameAndSubIdFromGenericKey(TagFieldKey genericKey)
+    protected FrameAndSubId getFrameAndSubIdFromGenericKey(FieldKey genericKey)
     {
         ID3v23FieldKey id3v23FieldKey = ID3v23Frames.getInstanceOf().getId3KeyFromGenericKey(genericKey);
         if (id3v23FieldKey == null)
@@ -906,28 +930,30 @@ public class ID3v23Tag extends AbstractID3v2Tag
         return ID3v23Frames.getInstanceOf();
     }
 
-     /**
-     *
+    /**
      * @return comparator used to order frames in preferred order for writing to file
-     * so that most important frames are written first.
+     *         so that most important frames are written first.
      */
     public Comparator getPreferredFrameOrderComparator()
     {
         return ID3v23PreferredFrameOrderComparator.getInstanceof();
     }
 
-     public List<Artwork> getArtworkList()
+    /**
+     * {@inheritDoc}
+     */
+    public List<Artwork> getArtworkList()
     {
-        List<TagField> coverartList = get(TagFieldKey.COVER_ART);
-        List<Artwork> artworkList   = new ArrayList<Artwork>(coverartList.size());
+        List<TagField> coverartList = getFields(FieldKey.COVER_ART);
+        List<Artwork> artworkList = new ArrayList<Artwork>(coverartList.size());
 
-        for(TagField next:coverartList)
+        for (TagField next : coverartList)
         {
             FrameBodyAPIC coverArt = (FrameBodyAPIC) ((AbstractID3v2Frame) next).getBody();
             Artwork artwork = new Artwork();
             artwork.setMimeType(coverArt.getMimeType());
             artwork.setPictureType(coverArt.getPictureType());
-            if(coverArt.isImageUrl())
+            if (coverArt.isImageUrl())
             {
                 artwork.setLinked(true);
                 artwork.setImageUrl(coverArt.getImageUrl());
@@ -941,27 +967,31 @@ public class ID3v23Tag extends AbstractID3v2Tag
         return artworkList;
     }
 
-     public TagField createArtworkField(Artwork artwork) throws FieldDataInvalidException
+    /**
+     * {@inheritDoc}
+     */    
+    public TagField createField(Artwork artwork) throws FieldDataInvalidException
     {
-        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(TagFieldKey.COVER_ART).getFrameId());
+        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(FieldKey.COVER_ART).getFrameId());
         FrameBodyAPIC body = (FrameBodyAPIC) frame.getBody();
-        body.setObjectValue(DataTypes.OBJ_PICTURE_DATA,artwork.getBinaryData());
+        body.setObjectValue(DataTypes.OBJ_PICTURE_DATA, artwork.getBinaryData());
         body.setObjectValue(DataTypes.OBJ_PICTURE_TYPE, artwork.getPictureType());
         body.setObjectValue(DataTypes.OBJ_MIME_TYPE, artwork.getMimeType());
         body.setObjectValue(DataTypes.OBJ_DESCRIPTION, "");
         return frame;
     }
 
-     /**
+    /**
      * Create Artwork
      *
      * @param data
      * @param mimeType of the image
      * @see PictureTypes
+     * @return
      */
     public TagField createArtworkField(byte[] data, String mimeType)
     {
-        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(TagFieldKey.COVER_ART).getFrameId());
+        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(FieldKey.COVER_ART).getFrameId());
         FrameBodyAPIC body = (FrameBodyAPIC) frame.getBody();
         body.setObjectValue(DataTypes.OBJ_PICTURE_DATA, data);
         body.setObjectValue(DataTypes.OBJ_PICTURE_TYPE, PictureTypes.DEFAULT_ID);
@@ -969,4 +999,9 @@ public class ID3v23Tag extends AbstractID3v2Tag
         body.setObjectValue(DataTypes.OBJ_DESCRIPTION, "");
         return frame;
     }
+
+    public int getPaddingSize()
+    {
+        return paddingSize;
+    }
 }
