@@ -16,17 +16,12 @@
 package org.jaudiotagger.tag.id3;
 
 import org.jaudiotagger.FileConstants;
-import org.jaudiotagger.logging.FileSystemMessage;
-import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.audio.mp3.MP3File;
-import org.jaudiotagger.audio.exceptions.UnableToModifyFileException;
-import org.jaudiotagger.audio.exceptions.UnableToCreateFileException;
+import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.tag.*;
-import org.jaudiotagger.tag.mp4.field.Mp4TagCoverField;
 import org.jaudiotagger.tag.datatype.Artwork;
 import org.jaudiotagger.tag.datatype.DataTypes;
 import org.jaudiotagger.tag.id3.framebody.*;
-import org.jaudiotagger.tag.id3.valuepair.ImageFormats;
 import org.jaudiotagger.tag.lyrics3.AbstractLyrics3;
 import org.jaudiotagger.tag.lyrics3.Lyrics3v2;
 import org.jaudiotagger.tag.lyrics3.Lyrics3v2Field;
@@ -35,11 +30,7 @@ import org.jaudiotagger.tag.reference.PictureTypes;
 
 import java.io.File;
 import java.io.IOException;
-import java.io.RandomAccessFile;
-import java.io.FileNotFoundException;
 import java.nio.ByteBuffer;
-import java.nio.channels.FileChannel;
-import java.nio.channels.FileLock;
 import java.nio.channels.WritableByteChannel;
 import java.util.*;
 import java.util.logging.Level;
@@ -49,7 +40,7 @@ import java.util.logging.Level;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v24Tag.java,v 1.51 2008/12/10 13:14:27 paultaylor Exp $
+ * @version $Id: ID3v24Tag.java,v 1.57 2009/11/12 15:42:58 paultaylor Exp $
  */
 public class ID3v24Tag extends AbstractID3v2Tag
 {
@@ -222,22 +213,40 @@ public class ID3v24Tag extends AbstractID3v2Tag
     protected boolean tagRestriction = false;
 
     /**
+     * If Set Image encoding restrictions
      *
+     *  0   No restrictions
+     *  1   Images are encoded only with PNG [PNG] or JPEG [JFIF].
      */
     protected byte imageEncodingRestriction = 0;
 
     /**
+     * If set Image size restrictions
      *
+     *  00  No restrictions
+     *  01  All images are 256x256 pixels or smaller.
+     *  10  All images are 64x64 pixels or smaller.
+     *  11  All images are exactly 64x64 pixels, unless required
+     *      otherwise.
      */
     protected byte imageSizeRestriction = 0;
 
     /**
+     * If set then Tag Size Restrictions
      *
+     *  00   No more than 128 frames and 1 MB total tag size.
+     *  01   No more than 64 frames and 128 KB total tag size.
+     *  10   No more than 32 frames and 40 KB total tag size.
+     *  11   No more than 32 frames and 4 KB total tag size.
      */
     protected byte tagSizeRestriction = 0;
 
     /**
+     *  If set Text encoding restrictions
      *
+     *  0    No restrictions
+     *  1    Strings are only encoded with ISO-8859-1 [ISO-8859-1] or
+     *       UTF-8 [UTF-8].
      */
     protected byte textEncodingRestriction = 0;
 
@@ -248,7 +257,17 @@ public class ID3v24Tag extends AbstractID3v2Tag
 
 
     /**
+     *  If set Text fields size restrictions
      *
+     *  00   No restrictions
+     *  01   No string is longer than 1024 characters.
+     *   10   No string is longer than 128 characters.
+     *  11   No string is longer than 30 characters.
+     *
+     *  Note that nothing is said about how many bytes is used to
+     *  represent those characters, since it is encoding dependent. If a
+     *  text frame consists of more than one string, the sum of the
+     *  strungs is restricted as stated.
      */
     protected byte textFieldSizeRestriction = 0;
 
@@ -312,68 +331,122 @@ public class ID3v24Tag extends AbstractID3v2Tag
         }
     }
 
-    /**
-     * Copy frames from one tag into a v2.4 tag
-     */
-    protected void copyFrames(AbstractID3v2Tag copyObject)
+    protected void addFrame(AbstractID3v2Frame frame)
     {
-        logger.info("Copying Frames,there are:" + copyObject.frameMap.keySet().size() + " different types");
-        frameMap = new LinkedHashMap();
-        //Copy Frames that are a valid 2.4 type
-        Iterator iterator = copyObject.frameMap.keySet().iterator();
-        AbstractID3v2Frame frame;
-        ID3v24Frame newFrame = null;
-        while (iterator.hasNext())
+        try
         {
-            String id = (String) iterator.next();
-            Object o = copyObject.frameMap.get(id);
-            //SingleFrames
-            if (o instanceof AbstractID3v2Frame)
-            {
-                frame = (AbstractID3v2Frame) o;
-                try
-                {
-                    newFrame = new ID3v24Frame(frame);
-                    logger.info("Adding Frame:" + newFrame.getIdentifier());
-                    copyFrameIntoMap(newFrame.getIdentifier(), newFrame);
-                }
-                catch (InvalidFrameException ife)
-                {
-                    logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier(), ife);
-                }
-            }
-            //MultiFrames
-            else if (o instanceof ArrayList)
-            {
-                ArrayList<AbstractID3v2Frame> multiFrame = new ArrayList<AbstractID3v2Frame>();
-                for (ListIterator<AbstractID3v2Frame> li = ((ArrayList<AbstractID3v2Frame>) o).listIterator(); li.hasNext();)
-                {
-                    frame = li.next();
-                    try
-                    {
-                        newFrame = new ID3v24Frame(frame);
-                        multiFrame.add(newFrame);
-                    }
-                    catch (InvalidFrameException ife)
-                    {
-                        logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier());
-                    }
-                }
-                //Ensure that the list actually contains at lest one value before adding
-                if(multiFrame.size()>0)
-                {
-                    if (newFrame != null)
-                    {
-                        logger.finest("Adding multi frame list to map:" + newFrame.getIdentifier());
-                        frameMap.put(newFrame.getIdentifier(), multiFrame);
-                    }
-                }
-            }
+            ID3v24Frame newFrame = new ID3v24Frame(frame);
+            copyFrameIntoMap(newFrame.getIdentifier(), newFrame);
+        }
+        catch (InvalidFrameException ife)
+        {
+            logger.log(Level.SEVERE, "Unable to convert frame:" + frame.getIdentifier());
         }
     }
 
+    /*
+       * Copy framne into map, whilst accounting for multiple frame sof same type which can occur even if there were
+       * not frames of the dame type in the original tag
+       *
+       * The frame already exists this shouldnt normally happen because frames
+       * that are allowed to be multiple don't call this method. Frames that
+       * arent allowed to be multiple aren't added to hashmap in first place when
+       * originally added.
+       *
+       * We only want to allow one of the frames going forward but we try and merge
+       * all the information into the one frame. However there is a problem here that
+       * if we then take this, modify it and try to write back the original values
+       * we could lose some information although this info is probably invalid anyway.
+       *
+       * However converting some frames from tag of one version to another may
+       * mean that two different frames both get converted to one frame, this
+       * particulary applies to DateTime fields which were originally two fields
+       * in v2.3 but are one field in v2.4.
+       */
+       @Override
+       protected void copyFrameIntoMap(String id, AbstractID3v2Frame newFrame)
+       {
+
+           if (frameMap.containsKey(newFrame.getIdentifier()))
+           {
+               Object o = frameMap.get(newFrame.getIdentifier());
+               if(o instanceof AbstractID3v2Frame)
+               {
+                   //Retrieve the frame with the same id we have already loaded into the map
+                   AbstractID3v2Frame firstFrame = (AbstractID3v2Frame) frameMap.get(newFrame.getIdentifier());
+
+
+                   //Two different frames both converted to TDRCFrames, now if this is the case one of them
+                   //may have actually have been created as a FrameUnsupportedBody because TDRC is only
+                   //supported in ID3v24, but is often created in v23 tags as well together with the valid TYER
+                   //frame
+                   if (newFrame.getBody() instanceof FrameBodyTDRC)
+                   {
+                       if (firstFrame.getBody() instanceof FrameBodyTDRC)
+                       {
+                           logger.finest("Modifying frame in map:" + newFrame.getIdentifier());
+                           FrameBodyTDRC body = (FrameBodyTDRC) firstFrame.getBody();
+                           FrameBodyTDRC newBody = (FrameBodyTDRC) newFrame.getBody();
+
+                           //#304:Check for NullPointer, just ignore this frame
+                           if(newBody.getOriginalID()==null)
+                           {
+                               return;
+                           }
+                           //Just add the data to the frame
+                           if (newBody.getOriginalID().equals(ID3v23Frames.FRAME_ID_V3_TYER))
+                           {
+                               body.setYear(newBody.getText());
+                           }
+                           else if (newBody.getOriginalID().equals(ID3v23Frames.FRAME_ID_V3_TDAT))
+                           {
+                               body.setDate(newBody.getText());
+                           }
+                           else if (newBody.getOriginalID().equals(ID3v23Frames.FRAME_ID_V3_TIME))
+                           {
+                               body.setTime(newBody.getText());
+                           }
+                           else if (newBody.getOriginalID().equals(ID3v23Frames.FRAME_ID_V3_TRDA))
+                           {
+                               body.setReco(newBody.getText());
+                           }
+                       }
+                       // The first frame was a TDRC frame that was not really allowed, this new frame was probably a
+                       // valid frame such as TYER which has been converted to TDRC, replace the firstframe with this frame
+                       else if (firstFrame.getBody() instanceof FrameBodyUnsupported)
+                       {
+                           frameMap.put(newFrame.getIdentifier(), newFrame);
+                       }
+                       else
+                       {
+                           //we just lose this frame, weve already got one with the correct id.
+                           //TODO may want to store this somewhere
+                           logger.warning("Found duplicate TDRC frame in invalid situation,discarding:" + newFrame.getIdentifier());
+                       }
+                   }
+                   else
+                   {
+                       List<AbstractID3v2Frame> list = new ArrayList<AbstractID3v2Frame>();
+                       list.add(firstFrame);
+                       list.add(newFrame);
+                       frameMap.put(newFrame.getIdentifier(), list);
+                   }
+               }
+               else
+               {
+                    List<AbstractID3v2Frame> list = (List)o;
+                    list.add(newFrame);
+               }
+           }
+           else
+           {
+               frameMap.put(newFrame.getIdentifier(), newFrame);
+           }
+       }
+
     /**
      * Copy Constructor, creates a new ID3v2_4 Tag based on another ID3v2_4 Tag
+     * @param copyObject
      */
     public ID3v24Tag(ID3v24Tag copyObject)
     {
@@ -550,18 +623,18 @@ public class ID3v24Tag extends AbstractID3v2Tag
         int size = TAG_HEADER_LENGTH;
         if (extended)
         {
-            size += this.TAG_EXT_HEADER_LENGTH;
+            size += TAG_EXT_HEADER_LENGTH;
             if (updateTag)
             {
-                size += this.TAG_EXT_HEADER_UPDATE_LENGTH;
+                size += TAG_EXT_HEADER_UPDATE_LENGTH;
             }
             if (crcDataFlag)
             {
-                size += this.TAG_EXT_HEADER_CRC_LENGTH;
+                size += TAG_EXT_HEADER_CRC_LENGTH;
             }
             if (tagRestriction)
             {
-                size += this.TAG_EXT_HEADER_RESTRICTION_LENGTH;
+                size += TAG_EXT_HEADER_RESTRICTION_LENGTH;
             }
         }
         size += super.getSize();
@@ -575,7 +648,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
      */
     public boolean equals(Object obj)
     {
-        if ((obj instanceof ID3v24Tag) == false)
+        if (!(obj instanceof ID3v24Tag))
         {
             return false;
         }
@@ -608,11 +681,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
         {
             return false;
         }
-        if (this.updateTag != object.updateTag)
-        {
-            return false;
-        }
-        return super.equals(obj);
+        return this.updateTag == object.updateTag && super.equals(obj);
     }
 
     /**
@@ -626,7 +695,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
     {
 
         //Skip over flags
-        byte flags = buffer.get();
+        buffer.get();
 
         // Read the size, this is size of tag not including  the tag header
         int size = ID3SyncSafeInteger.bufferToValue(buffer);
@@ -636,22 +705,15 @@ public class ID3v24Tag extends AbstractID3v2Tag
     }
 
     /**
-     * Read Tag from Specified file.
-     * Read tag header, delegate reading of frames to readFrames()
+     * Read header flags
+     * <p/>
+     * <p>Log info messages for falgs that have been set and log warnings when bits have been set for unknown flags</p>
      *
-     * @param byteBuffer to read the tag from
+     * @param byteBuffer
      * @throws TagException
-     * @throws TagNotFoundException
-     * @throws InvalidTagException
      */
-    public void read(ByteBuffer byteBuffer) throws TagException
+    private void readHeaderFlags(ByteBuffer byteBuffer) throws TagException
     {
-        int size;
-        byte[] buffer;
-        if (seek(byteBuffer) == false)
-        {
-            throw new TagNotFoundException(getLoggingFilename() + ":" + getIdentifier() + " tag not found");
-        }
         //Flags
         byte flags = byteBuffer.get();
         unsynchronization = (flags & MASK_V24_UNSYNCHRONIZATION) != 0;
@@ -659,85 +721,148 @@ public class ID3v24Tag extends AbstractID3v2Tag
         experimental = (flags & MASK_V24_EXPERIMENTAL) != 0;
         footer = (flags & MASK_V24_FOOTER_PRESENT) != 0;
 
+        //Not allowable/Unknown Flags
+        if ((flags & FileConstants.BIT3) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT3));
+        }
+
+        if ((flags & FileConstants.BIT2) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT2));
+        }
+
+        if ((flags & FileConstants.BIT1) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT1));
+        }
+
+        if ((flags & FileConstants.BIT0) != 0)
+        {
+            logger.warning(ErrorMessage.ID3_INVALID_OR_UNKNOWN_FLAG_SET.getMsg(getLoggingFilename(), FileConstants.BIT0));
+        }
+
+
         if (isUnsynchronization())
         {
-            logger.info(getLoggingFilename() + ":" + "ID3v24 Tag is unsynchronized");
+            logger.info(ErrorMessage.ID3_TAG_UNSYNCHRONIZED.getMsg(getLoggingFilename()));
         }
 
         if (extended)
         {
-            logger.warning(getLoggingFilename() + ":" + "ID3v24 Tag is extended");
+            logger.info(ErrorMessage.ID3_TAG_EXTENDED.getMsg(getLoggingFilename()));
         }
 
         if (experimental)
         {
-            logger.warning(getLoggingFilename() + ":" + "ID3v24 Tag is experimental");
+            logger.info(ErrorMessage.ID3_TAG_EXPERIMENTAL.getMsg(getLoggingFilename()));
         }
 
         if (footer)
         {
-            logger.warning(getLoggingFilename() + ":" + "ID3v24 Tag has footer");
+            logger.warning(ErrorMessage.ID3_TAG_FOOTER.getMsg(getLoggingFilename()));
         }
+    }
+
+    /**
+     * Read the optional extended header
+     *
+     * @param byteBuffer
+     * @param size
+     * @throws org.jaudiotagger.tag.InvalidTagException
+     */
+    private void readExtendedHeader(ByteBuffer byteBuffer, int size) throws InvalidTagException
+    {
+        byte[] buffer;
+
+        // int is 4 bytes.
+        int extendedHeaderSize = byteBuffer.getInt();
+
+        // the extended header must be at least 6 bytes
+        if (extendedHeaderSize <= TAG_EXT_HEADER_LENGTH)
+        {
+            throw new InvalidTagException(ErrorMessage.ID3_EXTENDED_HEADER_SIZE_TOO_SMALL.getMsg(getLoggingFilename(), extendedHeaderSize));
+        }
+
+        //Number of bytes
+        byteBuffer.get();
+
+        // Read the extended flag bytes
+        byte extFlag = byteBuffer.get();
+        updateTag       = (extFlag & MASK_V24_TAG_UPDATE)       != 0;
+        crcDataFlag     = (extFlag & MASK_V24_CRC_DATA_PRESENT) != 0;
+        tagRestriction  = (extFlag & MASK_V24_TAG_RESTRICTIONS) != 0;
+
+        // read the length byte if the flag is set
+        // this tag should always be zero but just in case
+        // read this information.
+        if (updateTag)
+        {
+            byteBuffer.get();
+        }
+
+        //CRC-32
+        if (crcDataFlag)
+        {
+            // the CRC has a variable length
+            byteBuffer.get();
+            buffer = new byte[TAG_EXT_HEADER_CRC_DATA_LENGTH];
+            byteBuffer.get(buffer, 0, TAG_EXT_HEADER_CRC_DATA_LENGTH);
+            crcData = 0;
+            for (int i = 0; i < TAG_EXT_HEADER_CRC_DATA_LENGTH; i++)
+            {
+                crcData <<= 8;
+                crcData += buffer[i];
+            }
+        }
+
+        //Tag Restriction
+        if (tagRestriction)
+        {
+            byteBuffer.get();
+            buffer = new byte[1];
+            byteBuffer.get(buffer, 0, 1);
+            tagSizeRestriction          = (byte) ((buffer[0] & MASK_V24_TAG_SIZE_RESTRICTIONS) >> 6);
+            textEncodingRestriction     = (byte) ((buffer[0] & MASK_V24_TEXT_ENCODING_RESTRICTIONS) >> 5);
+            textFieldSizeRestriction    = (byte) ((buffer[0] & MASK_V24_TEXT_FIELD_SIZE_RESTRICTIONS) >> 3);
+            imageEncodingRestriction    = (byte) ((buffer[0] & MASK_V24_IMAGE_ENCODING) >> 2);
+            imageSizeRestriction        = (byte) (buffer[0] & MASK_V24_IMAGE_SIZE_RESTRICTIONS);
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void read(ByteBuffer byteBuffer) throws TagException
+    {
+        int size;
+        byte[] buffer;
+        if (!seek(byteBuffer))
+        {
+            throw new TagNotFoundException(getLoggingFilename() + ":" + getIdentifier() + " tag not found");
+        }
+        logger.info(getLoggingFilename() + ":" + "Reading ID3v24 tag");
+        readHeaderFlags(byteBuffer);
 
         // Read the size, this is size of tag apart from tag header
         size = ID3SyncSafeInteger.bufferToValue(byteBuffer);
         logger.info(getLoggingFilename() + ":" + "Reading tag from file size set in header is" + size);
+
         if (extended)
         {
-            // int is 4 bytes.
-            int extendedHeaderSize = byteBuffer.getInt();
-            // the extended header must be atleast 6 bytes
-            if (extendedHeaderSize <= TAG_EXT_HEADER_LENGTH)
-            {
-                throw new InvalidTagException(getLoggingFilename() + ":" + "Invalid Extended Header Size.");
-            }
-            //Number of bytes
-            byteBuffer.get();
-            // Read the extended flag bytes
-            byte extFlag = byteBuffer.get();
-            updateTag = (extFlag & MASK_V24_TAG_UPDATE) != 0;
-            crcDataFlag = (extFlag & MASK_V24_CRC_DATA_PRESENT) != 0;
-            tagRestriction = (extFlag & MASK_V24_TAG_RESTRICTIONS) != 0;
-            // read the length byte if the flag is set
-            // this tag should always be zero but just in case
-            // read this information.
-            if (updateTag)
-            {
-                byteBuffer.get();
-            }
-            if (crcDataFlag)
-            {
-                // the CRC has a variable length
-                byteBuffer.get();
-                buffer = new byte[TAG_EXT_HEADER_CRC_DATA_LENGTH];
-                byteBuffer.get(buffer, 0, TAG_EXT_HEADER_CRC_DATA_LENGTH);
-                crcData = 0;
-                for (int i = 0; i < TAG_EXT_HEADER_CRC_DATA_LENGTH; i++)
-                {
-                    crcData <<= 8;
-                    crcData += buffer[i];
-                }
-            }
-            if (tagRestriction)
-            {
-                byteBuffer.get();
-                buffer = new byte[1];
-                byteBuffer.get(buffer, 0, 1);
-                tagSizeRestriction = (byte) ((buffer[0] & MASK_V24_TAG_SIZE_RESTRICTIONS) >> 6);
-                textEncodingRestriction = (byte) ((buffer[0] & MASK_V24_TEXT_ENCODING_RESTRICTIONS) >> 5);
-                textFieldSizeRestriction = (byte) ((buffer[0] & MASK_V24_TEXT_FIELD_SIZE_RESTRICTIONS) >> 3);
-                imageEncodingRestriction = (byte) ((buffer[0] & MASK_V24_IMAGE_ENCODING) >> 2);
-                imageSizeRestriction = (byte) (buffer[0] & MASK_V24_IMAGE_SIZE_RESTRICTIONS);
-            }
+            readExtendedHeader(byteBuffer, size);
         }
+
         //Note if there was an extended header the size value has padding taken
         //off so we dont search it.
         readFrames(byteBuffer, size);
-
     }
 
     /**
      * Read frames from tag
+     * @param byteBuffer
+     * @param size
      */
     protected void readFrames(ByteBuffer byteBuffer, int size)
     {
@@ -844,18 +969,18 @@ public class ID3v24Tag extends AbstractID3v2Tag
         int additionalHeaderSize = 0;
         if (extended)
         {
-            additionalHeaderSize += this.TAG_EXT_HEADER_LENGTH;
+            additionalHeaderSize += TAG_EXT_HEADER_LENGTH;
             if (updateTag)
             {
-                additionalHeaderSize += this.TAG_EXT_HEADER_UPDATE_LENGTH;
+                additionalHeaderSize += TAG_EXT_HEADER_UPDATE_LENGTH;
             }
             if (crcDataFlag)
             {
-                additionalHeaderSize += this.TAG_EXT_HEADER_CRC_LENGTH;
+                additionalHeaderSize += TAG_EXT_HEADER_CRC_LENGTH;
             }
             if (tagRestriction)
             {
-                additionalHeaderSize += this.TAG_EXT_HEADER_RESTRICTION_LENGTH;
+                additionalHeaderSize += TAG_EXT_HEADER_RESTRICTION_LENGTH;
             }
         }
 
@@ -931,11 +1056,9 @@ public class ID3v24Tag extends AbstractID3v2Tag
     }
 
     /**
-     * Write this tag to file.
-     *
-     * @param file
-     * @throws IOException
+     * {@inheritDoc}
      */
+    @Override
     public void write(File file, long audioStartLocation) throws IOException
     {
         logger.info("Writing tag to file");
@@ -950,15 +1073,13 @@ public class ID3v24Tag extends AbstractID3v2Tag
         int padding = sizeIncPadding - (bodyByteBuffer.length + TAG_HEADER_LENGTH);
 
         ByteBuffer headerBuffer = writeHeaderToBuffer(padding, bodyByteBuffer.length);
-        writeBufferToFile(file,headerBuffer, bodyByteBuffer,padding,sizeIncPadding,audioStartLocation);       
+        writeBufferToFile(file, headerBuffer, bodyByteBuffer, padding, sizeIncPadding, audioStartLocation);
     }
 
     /**
-     * Write tag to channel
-     *
-     * @param channel
-     * @throws IOException
+     * {@inheritDoc}
      */
+    @Override
     public void write(WritableByteChannel channel) throws IOException
     {
         logger.info("Writing tag to channel");
@@ -1088,6 +1209,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
      *
      * @param id3v24FieldKey
      * @return
+     * @throws org.jaudiotagger.tag.KeyNotFoundException
      */
     public String getFirst(ID3v24FieldKey id3v24FieldKey) throws KeyNotFoundException
     {
@@ -1095,7 +1217,32 @@ public class ID3v24Tag extends AbstractID3v2Tag
         {
             throw new KeyNotFoundException();
         }
-        return super.doGetFirst(new FrameAndSubId(id3v24FieldKey.getFrameId(), id3v24FieldKey.getSubId()));
+
+        FrameAndSubId frameAndSubId = new FrameAndSubId(id3v24FieldKey.getFrameId(), id3v24FieldKey.getSubId());
+        if (id3v24FieldKey == ID3v24FieldKey.TRACK)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTRCK)frame.getBody()).getTrackNo());
+        }
+        else if (id3v24FieldKey == ID3v24FieldKey.TRACK_TOTAL)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTRCK)frame.getBody()).getTrackTotal());
+        }
+        else if (id3v24FieldKey == ID3v24FieldKey.DISC_NO)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTPOS)frame.getBody()).getDiscNo());
+        }
+        else if (id3v24FieldKey == ID3v24FieldKey.DISC_TOTAL)
+        {
+            AbstractID3v2Frame frame = getFirstField(frameAndSubId.getFrameId());
+            return String.valueOf(((FrameBodyTPOS)frame.getBody()).getDiscTotal());
+        }
+        else
+        {
+            return super.doGetFirst(frameAndSubId);
+        }
     }
 
 
@@ -1103,6 +1250,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
      * Delete fields with this id3v24FieldKey
      *
      * @param id3v24FieldKey
+     * @throws org.jaudiotagger.tag.KeyNotFoundException
      */
     public void deleteTagField(ID3v24FieldKey id3v24FieldKey) throws KeyNotFoundException
     {
@@ -1114,7 +1262,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
     }
 
 
-    protected FrameAndSubId getFrameAndSubIdFromGenericKey(TagFieldKey genericKey)
+    protected FrameAndSubId getFrameAndSubIdFromGenericKey(FieldKey genericKey)
     {
         ID3v24FieldKey id3v24FieldKey = ID3v24Frames.getInstanceOf().getId3KeyFromGenericKey(genericKey);
         if (id3v24FieldKey == null)
@@ -1129,28 +1277,27 @@ public class ID3v24Tag extends AbstractID3v2Tag
         return ID3v24Frames.getInstanceOf();
     }
 
-     /**
-     *
+    /**
      * @return comparator used to order frames in preferred order for writing to file
-     * so that most important frames are written first.
+     *         so that most important frames are written first.
      */
     public Comparator getPreferredFrameOrderComparator()
     {
         return ID3v24PreferredFrameOrderComparator.getInstanceof();
     }
 
-     public List<Artwork> getArtworkList()
+    public List<Artwork> getArtworkList()
     {
-        List<TagField> coverartList = get(TagFieldKey.COVER_ART);
-        List<Artwork> artworkList   = new ArrayList<Artwork>(coverartList.size());
+        List<TagField> coverartList = getFields(FieldKey.COVER_ART);
+        List<Artwork> artworkList = new ArrayList<Artwork>(coverartList.size());
 
-        for(TagField next:coverartList)
+        for (TagField next : coverartList)
         {
             FrameBodyAPIC coverArt = (FrameBodyAPIC) ((AbstractID3v2Frame) next).getBody();
             Artwork artwork = new Artwork();
             artwork.setMimeType(coverArt.getMimeType());
             artwork.setPictureType(coverArt.getPictureType());
-            if(coverArt.isImageUrl())
+            if (coverArt.isImageUrl())
             {
                 artwork.setLinked(true);
                 artwork.setImageUrl(coverArt.getImageUrl());
@@ -1164,11 +1311,11 @@ public class ID3v24Tag extends AbstractID3v2Tag
         return artworkList;
     }
 
-     public TagField createArtworkField(Artwork artwork) throws FieldDataInvalidException
+    public TagField createField(Artwork artwork) throws FieldDataInvalidException
     {
-        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(TagFieldKey.COVER_ART).getFrameId());
+        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(FieldKey.COVER_ART).getFrameId());
         FrameBodyAPIC body = (FrameBodyAPIC) frame.getBody();
-        body.setObjectValue(DataTypes.OBJ_PICTURE_DATA,artwork.getBinaryData());
+        body.setObjectValue(DataTypes.OBJ_PICTURE_DATA, artwork.getBinaryData());
         body.setObjectValue(DataTypes.OBJ_PICTURE_TYPE, artwork.getPictureType());
         body.setObjectValue(DataTypes.OBJ_MIME_TYPE, artwork.getMimeType());
         body.setObjectValue(DataTypes.OBJ_DESCRIPTION, "");
@@ -1181,10 +1328,11 @@ public class ID3v24Tag extends AbstractID3v2Tag
      * @param data
      * @param mimeType of the image
      * @see PictureTypes
+     * @return
      */
     public TagField createArtworkField(byte[] data, String mimeType)
     {
-        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(TagFieldKey.COVER_ART).getFrameId());
+        AbstractID3v2Frame frame = createFrame(getFrameAndSubIdFromGenericKey(FieldKey.COVER_ART).getFrameId());
         FrameBodyAPIC body = (FrameBodyAPIC) frame.getBody();
         body.setObjectValue(DataTypes.OBJ_PICTURE_DATA, data);
         body.setObjectValue(DataTypes.OBJ_PICTURE_TYPE, PictureTypes.DEFAULT_ID);
